@@ -103,10 +103,11 @@ class ArgumentParserWithKeywordHooks(ArgumentParser):
         if args and args[~0] and isinstance(args[~0], str):
             nestarg = args[~0]
             nesting = len(nestarg)
-            if nestarg == '[]':
+            if nestarg.startswith('[]'):
                 self.set_defaults(squeeze=True)
-                del args[~0:]
-            elif nestarg == ']' * nesting:
+                nestarg = nestarg[2:]
+                nesting = nesting - 2
+            if nestarg == ']' * nesting:
                 self.set_defaults(nesting=-nesting)
                 del args[~0:]
             elif nestarg == '[' * nesting:
