@@ -18,7 +18,7 @@ class pattern:
 
     def __init__(self, pattern):
         self.pattern = pattern
-        self.compiled = re.compile(bytes(self))
+        self.compiled = re.compile(B'(%s)' % self)
 
     def __bytes__(self):
         return str(self).encode('ascii')
@@ -91,7 +91,7 @@ class PatternEnum(enum.Enum):
         raise AttributeError
 
 
-__TLDS = R'(?i:{possible_tld})(?!(?:{dealbreakers}))'.format(
+_TLDS = R'(?i:{possible_tld})(?!(?:{dealbreakers}))'.format(
     possible_tld='|'.join(tlds),
     dealbreakers='|'.join([
         R'[a-z]',
@@ -102,79 +102,98 @@ __TLDS = R'(?i:{possible_tld})(?!(?:{dealbreakers}))'.format(
 )
 
 # see https://tools.ietf.org/html/rfc2181#section-11
-format_domain_normal = (
+_format_serrated_domain = (
     R'(?:[a-zA-Z0-9\_][a-zA-Z0-9\-\_]{{0,62}}?\.){repeat}'
     R'[a-zA-Z0-9\_][a-zA-Z0-9\-\_]{{1,62}}\.{tlds}'
 )
-format_domain_defang = (
+_format_defanged_domain = (
     R'(?:[a-zA-Z0-9\_][a-zA-Z0-9\-\_]{{0,62}}?(?:\[\.\]|\.)){repeat}'
     R'[a-zA-Z0-9\_][a-zA-Z0-9\-\_]{{1,62}}(?:\[\.\]|\.){tlds}'
 )
 
-pattern_domain = format_domain_normal.format(repeat='{0,20}', tlds=__TLDS)
 
-pattern_subdomain = format_domain_normal.format(repeat='{1,20}', tlds=__TLDS)
-pattern_domain_df = format_domain_defang.format(repeat='{0,20}', tlds=__TLDS)
+_pattern_serrated_domain = _format_serrated_domain.format(repeat='{0,20}', tlds=_TLDS)
+_pattern_defanged_domain = _format_defanged_domain.format(repeat='{0,20}', tlds=_TLDS)
 
-pattern_version = '|'.join('.' * k + 'version' + '.' * (10 - k) for k in range(10))
+_pattern_subdomain = _format_serrated_domain.format(repeat='{1,20}', tlds=_TLDS)
 
-pattern_octet = R'(?:1\d\d|2[1-4]\d|25[1-5]|[1-9]?\d)'
-pattern_ipv4 = R'(?<!\.|\d)(?:{o}\.){{3}}{o}(?![\d\.])'.format(o=pattern_octet)
-pattern_ipv4_df = R'(?:{o}{d}){{3}}{o}'.format(o=pattern_octet, d=R'(?:\[\.\]|\.)')
+_pattern_octet = R'(?:1\d\d|2[0-4]\d|25[0-5]|[1-9]?\d)'
+_pattern_serrated_ipv4 = R'(?<!\.|\d)(?:{o}\.){{3}}{o}(?![\d\.])'.format(o=_pattern_octet)
+_pattern_defanged_ipv4 = R'(?:{o}{d}){{3}}{o}'.format(o=_pattern_octet, d=R'(?:\[\.\]|\.)')
 
-pattern_socket = '(?:{ip}|{d})(?::\\d{{2,5}})'.format(ip=pattern_ipv4, d=pattern_domain)
-pattern_hostname = pattern_socket + '?'
-pattern_hostname_df = '(?:{ip}|{d})(?::\\d{{2,5}})?'.format(ip=pattern_ipv4_df, d=pattern_domain_df)
+# Taken from: https://stackoverflow.com/a/17871737/9130824
+_pattern_ipv6 = (
+    R'('
+    R'([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|'          # 1:2:3:4:5:6:7:8
+    R'([0-9a-fA-F]{1,4}:){1,7}:|'                         # 1::                              1:2:3:4:5:6:7::
+    R'([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|'         # 1::8             1:2:3:4:5:6::8  1:2:3:4:5:6::8
+    R'([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|'  # 1::7:8           1:2:3:4:5::7:8  1:2:3:4:5::8
+    R'([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|'  # 1::6:7:8         1:2:3:4::6:7:8  1:2:3:4::8
+    R'([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|'  # 1::5:6:7:8       1:2:3::5:6:7:8  1:2:3::8
+    R'([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|'  # 1::4:5:6:7:8     1:2::4:5:6:7:8  1:2::8
+    R'[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|'       # 1::3:4:5:6:7:8   1::3:4:5:6:7:8  1::8
+    R':((:[0-9a-fA-F]{1,4}){1,7}|:)|'                     # ::2:3:4:5:6:7:8  ::2:3:4:5:6:7:8 ::8       ::
+    R'fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|'     # fe80::7:8%eth0   fe80::7:8%1     (link-local IPv6 addresses with zone index)
+    R'::(ffff(:0{1,4}){0,1}:){0,1}'                       #
+    R'((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}'  #
+    R'(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|'          # ::255.255.255.255   ::ffff:255.255.255.255  ::ffff:0:255.255.255.255
+    R'([0-9a-fA-F]{1,4}:){1,4}:'                          # (IPv4-mapped IPv6 addresses and IPv4-translated addresses)
+    R'((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}'  #
+    R'(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])'           # 2001:db8:3:4::192.0.2.33  64:ff9b::192.0.2.33
+    R')'                                                  # (IPv4-Embedded IPv6 Address)
+)
 
-pattern_integer = '[-+]?(?:0[bB][01]+|0[xX][0-9a-fA-F]+|0[1-7][0-7]*|[1-9][0-9]*|0)(?![a-zA-Z0-9])'
-pattern_float = R'[-+]?[0-9]*\.?[0-9]+(?:[eE][-+]?[0-9]+)?'
-pattern_cmdstr = R'''(?:"(?:""|[^"])*"|'(?:''|[^'])*')'''
-pattern_ps1str = R'''(?:"(?:`.|""|[^"])*"|'(?:''|[^'])*')'''
-pattern_string = R'''(?:"(?:[^"\\]|\\.)*"|'(?:[^'\\]|\\.)*')'''
+_pattern_serrated_socket = '(?:{ip}|{d})(?::\\d{{2,5}})'.format(ip=_pattern_serrated_ipv4, d=_pattern_serrated_domain)
+_pattern_defanged_socket = '(?:{ip}|{d})(?::\\d{{2,5}})'.format(ip=_pattern_defanged_ipv4, d=_pattern_defanged_domain)
 
-pattern_vbe = R'''#@~\^[ -~]{6}==(?:.*?)[ -~]{6}==\^#~@'''
+_pattern_serrated_hostname = _pattern_serrated_socket + '?'
+_pattern_defanged_hostname = _pattern_defanged_socket + '?'
 
-pattern_url = ''.join([
-    R'([a-zA-Z]{2,20}?://'                    # scheme
-    R'(?:[^"\'\s\x00-\x20\x7E-\xFF]{1,256}?'  # username
-    R'(?::[^"\'\s\x00-\x20\x7E-\xFF]{0,256}?)?@)?',
-    pattern_socket + '?',
-    R'(?:[/?#](?:[~/_=?&.,\w\%\-](?![a-zA-Z]{2,20}://))*)?)'
-])
+_pattern_integer = '[-+]?(?:0[bB][01]+|0[xX][0-9a-fA-F]+|0[1-7][0-7]*|[1-9][0-9]*|0)(?![a-zA-Z0-9])'
+_pattern_float = R'[-+]?[0-9]*\.?[0-9]+(?:[eE][-+]?[0-9]+)?'
+_pattern_cmdstr = R'''(?:"(?:""|[^"])*"|'(?:''|[^'])*')'''
+_pattern_ps1str = R'''(?:@"\s*?[\r\n].*?[\r\n]"@|@'\s*?[\r\n].*?[\r\n]'@|"(?:`.|""|[^"])*"|'(?:''|[^'])*')'''
 
-pattern_url_df = ''.join([
-    R'([a-zA-Z]{2,20}?(?:\[:\]|:)//'          # scheme
-    R'(?:[^"\'\s\x00-\x20\x7E-\xFF]{1,256}?'  # username
-    R'(?::[^"\'\s\x00-\x20\x7E-\xFF]{0,256}?)?@)?',
-    pattern_socket + '?',
-    R'(?:[/?#](?:[/_=?&.,\w\%\-](?![a-zA-Z]{2,20}://))*)?)'
-])
+_pattern_string = R'''(?:"(?:[^"\\]|\\.)*"|'(?:[^'\\]|\\.)*')'''
 
-pattern_email = R'(?:[a-zA-Z0-9_\.\+\-]{{1,256}}?)@(?:{})'.format(pattern_domain)
-pattern_guid = R'(?:\b|\{)[0-9A-Fa-f]{8}(?:\-[0-9A-Fa-f]{4}){3}\-[0-9A-Fa-f]{12}(?:\}|\b)'
+_pattern_vbe = R'''#@~\^[ -~]{6}==(?:.*?)[ -~]{6}==\^#~@'''
 
-pattern_pathpart_nospace = R'[-\w+,.;@\]\[\^`~]+'  # R'[^/\\:"<>|\s\x7E-\xFF\x00-\x1F\xAD]+'
-pattern_win_path_element = R'(?:{n} ){{0,4}}{n}'.format(n=pattern_pathpart_nospace)
-pattern_nix_path_element = R'(?:{n} ){{0,1}}{n}'.format(n=pattern_pathpart_nospace)
-pattern_win_env_variable = R'%[a-zA-Z][a-zA-Z0-9_\-\(\)]{2,}%'
+_part_url_credentials = (
+    R'(?:(?P<url_username>[^"\'\s\x00-\x20\x7E-\xFF]{1,256})?'
+    R'(?::(?P<url_password>[^"\'\s\x00-\x20\x7E-\xFF]{0,256})?)?@)?'
+)
+_prefix_serrated_url = R'(?P<url_scheme>(?P<url_protocol>[a-zA-Z]{2,20})?:\/\/)' + _part_url_credentials
+_prefix_defanged_url = R'(?P<url_scheme>(?P<url_protocol>[a-zA-Z]{2,20})?(?:\[:\]|:)\/\/)' + _part_url_credentials
+_suffix_combined_url = R'(?P<url_path>[/?#](?:[#/=:;$!?&.,\w\+\%\-\*\'~@()](?![a-zA-Z]{2,20}://))*)?'
 
-pattern_win_path = R'(?:{s})(?P<pathsep>[\\\/])(?:{p}(?P=pathsep))*{p}\b'.format(
+_pattern_serrated_url = F'{_prefix_serrated_url}(?P<url_host>{_pattern_serrated_hostname}){_suffix_combined_url}'
+_pattern_defanged_url = F'{_prefix_defanged_url}(?P<url_host>{_pattern_defanged_hostname}){_suffix_combined_url}'
+
+_pattern_email = R'(?:[a-zA-Z0-9_\.\+\-]{{1,256}}?)@(?:{})'.format(_pattern_serrated_domain)
+_pattern_guid = R'(?:\b|\{)[0-9A-Fa-f]{8}(?:\-[0-9A-Fa-f]{4}){3}\-[0-9A-Fa-f]{12}(?:\}|\b)'
+
+_pattern_pathpart_nospace = R'[-\w+,.;@\]\[\^`~]+'  # R'[^/\\:"<>|\s\x7E-\xFF\x00-\x1F\xAD]+'
+_pattern_win_path_element = R'(?:{n} ){{0,4}}{n}'.format(n=_pattern_pathpart_nospace)
+_pattern_nix_path_element = R'(?:{n} ){{0,1}}{n}'.format(n=_pattern_pathpart_nospace)
+_pattern_win_env_variable = R'%[a-zA-Z][a-zA-Z0-9_\-\(\)]{2,}%'
+
+_pattern_win_path = R'(?:{s})(?P<pathsep>[\\\/])(?:{p}(?P=pathsep))*{p}\b'.format(
     s='|'.join([
-        pattern_win_env_variable,     # environment variable
+        _pattern_win_env_variable,     # environment variable
         R'[A-Za-z]:',                 # drive letter with colon
         R'\\\\[a-zA-Z0-9_.$]{1,50}',  # UNC path
         R'HK[A-Z_]{1,30}',            # registry root key
     ]),
-    p=pattern_win_path_element
+    p=_pattern_win_path_element
 )
 
-pattern_nix_path = R'\b/?(?:{n}/){{2,}}{n}\b'.format(n=pattern_nix_path_element)
-pattern_path = R'(?:{nix})|(?:{win})'.format(
-    nix=pattern_nix_path,
-    win=pattern_win_path
+_pattern_nix_path = R'\b/?(?:{n}/){{2,}}{n}\b'.format(n=_pattern_nix_path_element)
+_pattern_any_path = R'(?:{nix})|(?:{win})'.format(
+    nix=_pattern_nix_path,
+    win=_pattern_win_path
 )
 
-pattern_hexline = R'(?:{s}+\s+)?\s*{h}(?:\s+{s}+)?'.format(
+_pattern_hexline = R'(?:{s}+\s+)?\s*{h}(?:\s+{s}+)?'.format(
     h=tokenize(
         R'(?:0x)?[0-9a-f]{2}h?',
         sep=R'[- \t\/:;,\\]{1,3}'
@@ -182,7 +201,7 @@ pattern_hexline = R'(?:{s}+\s+)?\s*{h}(?:\s+{s}+)?'.format(
     s=R'[-\w:;,#\.\$\?!\/\\=\(\)\[\]\{\}]'
 )
 
-pattern_pem = (
+_pattern_pem = (
     R'-----BEGIN(?:\s[A-Z0-9]+)+-----{n}'
     R'(?:{b}{{40,100}}{n})+{b}{{1,100}}={{0,3}}{n}'
     R'-----END(?:\s[A-Z0-9]+)+-----'
@@ -202,28 +221,30 @@ class formats(PatternEnum):
     """
     An enumeration of patterns for certain formats.
     """
-    integer = pattern(pattern_integer)
+    integer = pattern(_pattern_integer)
     "Integer expressions"
-    float = pattern(pattern_float)
+    float = pattern(_pattern_float)
     "Floating point number expressions"
-    string = pattern(pattern_string)
+    string = pattern(_pattern_string)
     "C syntax string literal"
-    cmdstr = pattern(pattern_cmdstr)
+    cmdstr = pattern(_pattern_cmdstr)
     "Windows command line escaped string literal"
-    ps1str = pattern(pattern_ps1str)
+    ps1str = pattern(_pattern_ps1str)
     "PowerShell escaped string literal"
     printable = alphabet(R'[ -~]')
     "Any sequence of printable characters"
-    intarray = tokenize(pattern_integer, sep=R'\s*[;,]\s*', bound='')
+    intarray = tokenize(_pattern_integer, sep=R'\s*[;,]\s*', bound='')
     "Sequences of integers, separated by commas or semicolons"
     word = alphabet(R'\\w')
     "Sequences of word characters"
     alph = alphabet(R'[a-zA-Z]')
     "Sequences of alphabetic characters"
-    vbe = pattern(pattern_vbe)
+    vbe = pattern(_pattern_vbe)
     "Encoded Visual Basic Scripts"
     anum = alphabet(R'[a-zA-Z0-9]')
     "Sequences of alpha-numeric characters"
+    b32 = pattern('[A-Z2-7]+|[a-z2-7+]')
+    "Base32 encoded strings"
     b64 = alphabet(R'[0-9a-zA-Z\+\/]', postfix=R'[0-9a-zA-Z\+\/]{0,3}={0,3}')
     "Base64 encoded strings"
     b64u = alphabet(R'[0-9a-zA-Z\_\-]', postfix=R'[0-9a-zA-Z\_\-]{0,3}={0,3}')
@@ -232,7 +253,7 @@ class formats(PatternEnum):
     "Hexadecimal strings"
     HEX = alphabet(R'[0-9A-F]')
     "Uppercase hexadecimal strings"
-    hexdump = tokenize(pattern_hexline, bound='', sep=R'\s*\n')
+    hexdump = tokenize(_pattern_hexline, bound='', sep=R'\s*\n')
     """
     This pattern matches a typical hexdump output where hexadecimally encoded
     bytes are followed by a string which contains dots or printable characters
@@ -249,37 +270,39 @@ class indicators(PatternEnum):
     """
     An enumeration of patterns for indicators.
     """
-    domain = pattern(pattern_domain)
+    domain = pattern(_pattern_serrated_domain)
     "Domain names"
-    email = pattern(pattern_email)
+    email = pattern(_pattern_email)
     "Email addresses"
-    guid = pattern(pattern_guid)
+    guid = pattern(_pattern_guid)
     "Windows GUID strings"
-    ipv4 = pattern(pattern_ipv4)
+    ipv4 = pattern(_pattern_serrated_ipv4)
     "String representations of IPv4 addresses"
+    ipv6 = pattern(_pattern_ipv6)
+    "String representations of IPv6 addresses"
     md5 = alphabet('[0-9a-f]', at_least=32, at_most=32)
     "Hexadecimal strings of length 32"
     sha = alphabet('[0-9a-f]', at_least=40, at_most=40)
     "Hexadecimal strings of length 40"
     sha256 = alphabet('[0-9a-f]', at_least=64, at_most=64)
     "Hexadecimal strings of length 64"
-    hostname = pattern(pattern_hostname)
+    hostname = pattern(_pattern_serrated_hostname)
     "Any domain name or IPv4 address, optionally followd by a colon and a port number."
-    socket = pattern(pattern_socket)
+    socket = pattern(_pattern_serrated_socket)
     "Any domain name or IPv4 address followed by a colon and a (port) number"
-    subdomain = pattern(pattern_subdomain)
+    subdomain = pattern(_pattern_subdomain)
     "A domain which contains at least three parts, including the top level"
-    url = pattern(pattern_url)
+    url = pattern(_pattern_serrated_url)
     "Uniform resource locator addresses"
     btc = alphabet('[a-km-zA-HJ-NP-Z0-9]', prefix=R'(?<!\w)[13]', at_least=26, at_most=33)
     "Bitcoin addresses"
-    pem = pattern(pattern_pem)
+    pem = pattern(_pattern_pem)
     "A pattern matching PEM encoded cryptographic parameters"
     xmr = alphabet('[1-9A-HJ-NP-Za-km-z]', prefix='4[0-9AB]', at_least=90, at_most=120)
     "Monero addresses"
-    path = pattern(pattern_path)
+    path = pattern(_pattern_any_path)
     "Windows and Linux path names"
-    evar = pattern(pattern_win_env_variable)
+    evar = pattern(_pattern_win_env_variable)
     "Windows environment variables, i.e. something like `%APPDATA%`"
 
 
@@ -288,7 +311,7 @@ class defanged(PatternEnum):
     An enumeration of patterns for defanged indicators. Used only by the reverse
     operation of `refinery.defang`.
     """
-    hostname = pattern(pattern_hostname_df)
+    hostname = pattern(_pattern_defanged_hostname)
     "A defanged `refinery.lib.patterns.indicators.hostname`."
-    url = pattern(pattern_url_df)
+    url = pattern(_pattern_defanged_url)
     "A defanged `refinery.lib.patterns.indicators.url`."
