@@ -8,6 +8,9 @@ class rev(BlockTransformation):
     The blocks of the input data are output in reverse order. If the length of
     the input data is not a multiple of the block size, the data is truncated.
     """
+    def __init__(self, blocksize=1):
+        super().__init__(blocksize=blocksize)
+
     def process(self, data):
         if self.bytestream:
             return data[::-1]
