@@ -10,7 +10,7 @@ from .. import arg, Unit
 IMAGE_DIRECTORY_ENTRY_SECURITY = DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_SECURITY']
 
 
-def get_pe_size(pe: Union[PE, ByteString], overlay=True, sections=True, directories=True, certificate=True) -> int:
+def get_pe_size(pe: Union[PE, ByteString], overlay=True, sections=True, directories=True, certificate=True, memdump=False) -> int:
     """
     This fuction determines the size of a PE file, optionally taking into account the
     pefile module overlay computation, section information, data directory information,
@@ -26,6 +26,11 @@ def get_pe_size(pe: Union[PE, ByteString], overlay=True, sections=True, director
         for s in pe.sections
     ) or 0
 
+    memdump_value = memdump and max(
+        s.VirtualAddress + s.Misc_VirtualSize
+        for s in pe.sections
+    ) or 0
+
     directories_value = directories and max(
         pe.get_offset_from_rva(d.VirtualAddress) + d.Size
         for d in pe.OPTIONAL_HEADER.DATA_DIRECTORY
@@ -43,7 +48,8 @@ def get_pe_size(pe: Union[PE, ByteString], overlay=True, sections=True, director
         overlay_value,
         sections_value,
         directories_value,
-        cert_value
+        cert_value,
+        memdump_value
     )
 
 
