@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-from .. import Unit, RefineryCriticalException
+from .. import arg, Unit, RefineryCriticalException
 
 
 class sorted(Unit):
@@ -9,6 +9,9 @@ class sorted(Unit):
     This unit is a `refinery.nop` on single inputs.
     """
 
+    def __init__(self, length: arg.switch('-l', help='Sort items by length before sorting lexicographically.')):
+        super().__init__(length=length)
+
     def filter(self, chunks):
         sortbuffer = []
         invisibles = {}
@@ -21,7 +24,10 @@ class sorted(Unit):
             else:
                 sortbuffer.append(chunk)
 
-        sortbuffer.sort()
+        if self.args.length:
+            sortbuffer.sort(key=lambda t: (len(t), t))
+        else:
+            sortbuffer.sort()
 
         if not invisibles:
             yield from sortbuffer
