@@ -21,9 +21,9 @@ class lprefix(Unit):
         self,
         prefix: arg(nargs='?', type=str,
             help='Choose a Python format string to extract the prefix, default is "{default}".') = 'L',
-        limit : arg.number('-L', help='Only decode up to {varname} chunks and treat the rest as leftover data.') = INF,
+        count : arg.number('-c', help='Only decode up to {varname} chunks and treat the rest as leftover data.') = INF,
         strict: arg.switch('-S', help='Discard any leftover data with invalid length prefix.') = False,
-        single: arg.switch('-s', help='Equivalent to --strict --limit=1.') = False,
+        single: arg.switch('-s', help='Equivalent to --strict --count=1.') = False,
         header: arg('-H', action='count', default=0, help=(
             'Treat the parsed prefix as part of the body. Specify twice to strip the header afterwards.'
         )) = 0,
@@ -34,7 +34,7 @@ class lprefix(Unit):
     ):
         if single:
             strict = True
-            limit = 1
+            count = 1
         if not prefix:
             raise ValueError('an empty prefix was specified')
         if prefix[0] not in '<!=@>':
@@ -47,7 +47,7 @@ class lprefix(Unit):
             strict=strict,
             prefix=prefix,
             derive=derive,
-            limit=limit,
+            count=count,
             header=int(header)
         )
 
@@ -63,7 +63,7 @@ class lprefix(Unit):
             try:
                 while not mf.eof:
                     position = mf.tell()
-                    if count >= self.args.limit:
+                    if count >= self.args.count:
                         raise EOF
                     size = mf.read_struct(self.args.prefix, unwrap=True)
                     size = parse(N=size, **meta)
@@ -77,7 +77,7 @@ class lprefix(Unit):
                     yield mf.read(size)
                     count += 1
             except EOF as eof:
-                if self.args.strict or count >= self.args.limit:
+                if self.args.strict or count >= self.args.count:
                     return
                 if len(eof.rest) < size:
                     self.log_warn(F'attempted to read 0x{size:X} bytes, got only 0x{len(eof.rest):X}.')
